/** options for each client instance */
interface ClientOptions extends RequestInit {
    /** set the common root URL for all API requests */
    baseUrl?: string;
}
export interface BaseParams {
    path?: Record<string, unknown>;
    query?: Record<string, unknown>;
}
export type Method = 'get' | 'put' | 'post' | 'delete' | 'options' | 'head' | 'patch' | 'trace';
/** Gets a union of paths which have method */
type PathsWith<T, M extends Method> = {
    [Path in keyof T]: T[Path] extends {
        [K in M]: unknown;
    } ? Path : never;
}[keyof T];
type PathParams<T> = T extends {
    parameters: any;
} ? {
    params: T['parameters'];
} : {
    params?: BaseParams;
};
type MethodParams<T> = T extends {
    parameters: any;
} ? {
    params: T['parameters'];
} : {
    params?: BaseParams;
};
type Params<T> = PathParams<T> & MethodParams<T>;
type RequestBody<T> = T extends {
    requestBody: any;
} ? {
    body: Unwrap<T['requestBody']>;
} : {
    body?: never;
};
type FetchOptions<T> = Params<T> & RequestBody<T> & Omit<RequestInit, 'body'>;
type TruncatedResponse = Omit<Response, 'arrayBuffer' | 'blob' | 'body' | 'clone' | 'formData' | 'json' | 'text'>;
/** Infer request/response from content type */
type Unwrap<T> = T extends {
    content: {
        'application/json': any;
    };
} ? T['content']['application/json'] : T extends {
    content: {
        'application/json;charset=utf-8': any;
    };
} ? T['content']['application/json;charset=utf-8'] : T extends {
    content: {
        '*/*': any;
    };
} ? T['content']['*/*'] : T;
type Success<T> = T extends {
    200: any;
} ? T[200] : T extends {
    201: any;
} ? T[201] : T extends {
    202: any;
} ? T[202] : T extends {
    default: any;
} ? T['default'] : unknown;
type Error<T> = T extends {
    500: any;
} ? T[500] : T extends {
    404: any;
} ? T[404] : T extends {
    402: any;
} ? T[402] : T extends {
    401: any;
} ? T[401] : T extends {
    400: any;
} ? T[400] : T extends {
    422: any;
} ? T[422] : T extends {
    418: any;
} ? T[418] : T extends {
    417: any;
} ? T[417] : T extends {
    416: any;
} ? T[416] : T extends {
    415: any;
} ? T[415] : T extends {
    414: any;
} ? T[414] : T extends {
    413: any;
} ? T[413] : T extends {
    412: any;
} ? T[412] : T extends {
    411: any;
} ? T[411] : T extends {
    410: any;
} ? T[410] : T extends {
    409: any;
} ? T[409] : T extends {
    408: any;
} ? T[408] : T extends {
    407: any;
} ? T[407] : T extends {
    406: any;
} ? T[406] : T extends {
    405: any;
} ? T[405] : T extends {
    default: any;
} ? T['default'] : unknown;
type FetchResponse<T> = {
    data: T extends {
        responses: any;
    } ? NonNullable<Unwrap<Success<T['responses']>>> : unknown;
    error?: never;
    response: TruncatedResponse;
} | {
    data?: never;
    error: T extends {
        responses: any;
    } ? NonNullable<Unwrap<Error<T['responses']>>> : unknown;
    response: TruncatedResponse;
};
export default function createClient<T>(options?: ClientOptions): {
    /** Call a GET endpoint */
    get<U extends PathsWith<T, "get">, M extends keyof T[U]>(url: U, init: FetchOptions<T[U][M]>): Promise<FetchResponse<T[U][keyof T[U]]>>;
    /** Call a PUT endpoint */
    put<U_1 extends PathsWith<T, "put">, M_1 extends keyof T[U_1]>(url: U_1, init: FetchOptions<T[U_1][M_1]>): Promise<FetchResponse<T[U_1][keyof T[U_1]]>>;
    /** Call a POST endpoint */
    post<U_2 extends PathsWith<T, "post">, M_2 extends keyof T[U_2]>(url: U_2, init: FetchOptions<T[U_2][M_2]>): Promise<FetchResponse<T[U_2][keyof T[U_2]]>>;
    /** Call a DELETE endpoint */
    del<U_3 extends PathsWith<T, "delete">, M_3 extends keyof T[U_3]>(url: U_3, init: FetchOptions<T[U_3][M_3]>): Promise<FetchResponse<T[U_3][keyof T[U_3]]>>;
    /** Call a OPTIONS endpoint */
    options<U_4 extends PathsWith<T, "options">, M_4 extends keyof T[U_4]>(url: U_4, init: FetchOptions<T[U_4][M_4]>): Promise<FetchResponse<T[U_4][keyof T[U_4]]>>;
    /** Call a HEAD endpoint */
    head<U_5 extends PathsWith<T, "head">, M_5 extends keyof T[U_5]>(url: U_5, init: FetchOptions<T[U_5][M_5]>): Promise<FetchResponse<T[U_5][keyof T[U_5]]>>;
    /** Call a PATCH endpoint */
    patch<U_6 extends PathsWith<T, "patch">, M_6 extends keyof T[U_6]>(url: U_6, init: FetchOptions<T[U_6][M_6]>): Promise<FetchResponse<T[U_6][keyof T[U_6]]>>;
    /** Call a TRACE endpoint */
    trace<U_7 extends PathsWith<T, "trace">, M_7 extends keyof T[U_7]>(url: U_7, init: FetchOptions<T[U_7][M_7]>): Promise<FetchResponse<T[U_7][keyof T[U_7]]>>;
};
export {};
