const DEFAULT_HEADERS = {
    'Content-Type': 'application/json',
};
export default function createClient(options) {
    const defaultHeaders = new Headers({
        ...DEFAULT_HEADERS,
        ...(options?.headers ?? {}),
    });
    async function coreFetch(url, fetchOptions) {
        let { headers, body, params = {}, ...init } = fetchOptions || {};
        // URL
        let finalURL = `${options?.baseUrl ?? ''}${url}`;
        const { path, query } = params ?? {};
        if (path)
            for (const [k, v] of Object.entries(path))
                finalURL = finalURL.replace(`{${k}}`, encodeURIComponent(`${v}`.trim()));
        if (query)
            finalURL = `${finalURL}?${new URLSearchParams(query).toString()}`;
        // headers
        const baseHeaders = new Headers(defaultHeaders); // clone defaults (don’t overwrite!)
        const headerOverrides = new Headers(headers);
        for (const [k, v] of headerOverrides.entries()) {
            if (v === undefined || v === null)
                baseHeaders.delete(k); // allow `undefined` | `null` to erase value
            else
                baseHeaders.set(k, v);
        }
        // fetch!
        const res = await fetch(finalURL, {
            redirect: 'follow',
            ...options,
            ...init,
            headers: baseHeaders,
            body: typeof body === 'string' ? body : JSON.stringify(body),
        });
        const response = {
            bodyUsed: res.bodyUsed,
            headers: res.headers,
            ok: res.ok,
            redirected: res.redirected,
            status: res.status,
            statusText: res.statusText,
            type: res.type,
            url: res.url,
        };
        return res.ok ? { data: await res.json(), response } : { error: await res.json(), response };
    }
    return {
        /** Call a GET endpoint */
        async get(url, init) {
            return coreFetch(url, { ...init, method: 'GET' });
        },
        /** Call a PUT endpoint */
        async put(url, init) {
            return coreFetch(url, { ...init, method: 'PUT' });
        },
        /** Call a POST endpoint */
        async post(url, init) {
            return coreFetch(url, { ...init, method: 'POST' });
        },
        /** Call a DELETE endpoint */
        async del(url, init) {
            return coreFetch(url, { ...init, method: 'DELETE' });
        },
        /** Call a OPTIONS endpoint */
        async options(url, init) {
            return coreFetch(url, { ...init, method: 'OPTIONS' });
        },
        /** Call a HEAD endpoint */
        async head(url, init) {
            return coreFetch(url, { ...init, method: 'HEAD' });
        },
        /** Call a PATCH endpoint */
        async patch(url, init) {
            return coreFetch(url, { ...init, method: 'PATCH' });
        },
        /** Call a TRACE endpoint */
        async trace(url, init) {
            return coreFetch(url, { ...init, method: 'TRACE' });
        },
    };
}
